package com.aitrainer.api.controller

import com.aitrainer.api.model.CustomerExerciseDevice
import com.aitrainer.api.repository.CustomerExerciseDeviceRepository
import org.slf4j.LoggerFactory
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.access.annotation.Secured
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class CustomerExerciseDeviceController(private val customerExerciseDeviceRepository: CustomerExerciseDeviceRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @GetMapping("/customer_exercise_device/customer/{id}")
    fun getAllByCustomerId(@PathVariable(value = "id") customerId: Long): ResponseEntity<List<CustomerExerciseDevice>> {

        val deviceList  = customerExerciseDeviceRepository.findByCustomerId(customerId)
        logger.info("Get all devices by customerId $deviceList")

        return if(deviceList.isNotEmpty())
            ResponseEntity.ok().body(deviceList) else
            ResponseEntity.notFound().build()
    }

    @PostMapping("/customer_exercise_device")
    fun insertDevice(@Valid @RequestBody customerExerciseDevice: CustomerExerciseDevice): ResponseEntity<CustomerExerciseDevice> {
        val newCustomerExerciseDevice = customerExerciseDeviceRepository.save(customerExerciseDevice)
        logger.info("Create new device: $newCustomerExerciseDevice")
        return ResponseEntity.ok().body(newCustomerExerciseDevice)
    }

    @Secured
    @PostMapping("/customer_exercise_device/delete/{id}")
    fun deleteCustomerExerciseDevice(@PathVariable(value = "id") customerExerciseDeviceId: Long) :ResponseEntity<HttpStatus> {
        logger.info("Delete device by ID: $customerExerciseDeviceId")
        val returnDevice = customerExerciseDeviceRepository.findById(customerExerciseDeviceId).orElse(null)
            ?: return ResponseEntity.notFound().build()
        
        try {
            customerExerciseDeviceRepository.delete(returnDevice)
        } catch (e: IllegalArgumentException ) {
            return ResponseEntity.ok().body(HttpStatus.BAD_REQUEST)
        }

        return ResponseEntity.ok().body(HttpStatus.OK)
    }
}