package com.aitrainer.api.controller

import com.aitrainer.api.model.*
import com.aitrainer.api.repository.*
import com.google.gson.GsonBuilder
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/api")
class CustomerPackageController( private val customerRepository: CustomerRepository,
                                 private val customerExerciseDeviceRepository: CustomerExerciseDeviceRepository,
                                 private val exercisesRepository: ExercisesRepository,
                                 private val purchaseRepository: PurchaseRepository,
                                 private val customerPropertyRepository: CustomerPropertyRepository,
                                 private val exerciseResultRepository: ExerciseResultRepository,
                                 private val customerActivityRepository: CustomerActivityRepository,
                                 private val customerTrainingPlanRepository: CustomerTrainingPlanRepository,
) {

    @GetMapping("/app_customer_package/{id}")
    fun getCustomerPackageData(@PathVariable(value = "id") customerId: Long): ResponseEntity<String> {

        if (customerId <= 0) {
            return ResponseEntity.notFound().build()
        }

        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val customer: Customer = customerRepository.findById(customerId).orElse(null)
            ?: return ResponseEntity.notFound().build()
        val customerJson: String = gson.toJson(customer)

        val listCustomerExerciseDevices = customerExerciseDeviceRepository.findByCustomerId(customerId)
        val listCustomerExerciseDeviceJson = gson.toJson(listCustomerExerciseDevices)

        val listExercises = exercisesRepository.getAllByCustomerIdOrderByDateAddDesc(customerId)
        val listExercisesJson = gson.toJson(listExercises)

        val listPurchase = purchaseRepository.findByCustomerId(customerId)
        val listPurchaseJson = gson.toJson(listPurchase)

        val listCustomerPropertyAll = customerPropertyRepository.findAllByCustomerId(customerId)
        val listCustomerPropertyAllJson = gson.toJson(listCustomerPropertyAll)
        val listCustomerProperty = customerPropertyRepository.findLastPropertiesByCustomerId(customerId)
        val listCustomerPropertyJson = gson.toJson(listCustomerProperty)


        val listExerciseResult = exerciseResultRepository.getAllByCustomerId(customerId)
        val listExerciseResultJson = gson.toJson(listExerciseResult)

        val listActivityResult = customerActivityRepository.findByCustomerId(customerId)
        val listActivityJson = gson.toJson(listActivityResult)

        val listTrainingPlan = customerTrainingPlanRepository.findAllByCustomerId(customerId)
        val listTrainingPlanJson = gson.toJson(listTrainingPlan)

        val packageJson: String =
            getClassRecord(Customer::class.simpleName, customerJson) +
                    "|||" +  getClassRecord(CustomerExerciseDevice::class.simpleName, listCustomerExerciseDeviceJson) +
                    "|||" +  getClassRecord(Exercises::class.simpleName, listExercisesJson) +
                    "|||" +  getClassRecord(Purchase::class.simpleName, listPurchaseJson) +
                    "|||" +  getClassRecord(CustomerProperty::class.simpleName+"All", listCustomerPropertyAllJson) +
                    "|||" +  getClassRecord(CustomerProperty::class.simpleName, listCustomerPropertyJson) +
                    "|||" +  getClassRecord(ExerciseResult::class.simpleName, listExerciseResultJson+
                    "|||" +  getClassRecord(CustomerActivity::class.simpleName, listActivityJson) +
                    "|||" +  getClassRecord(CustomerTrainingPlan::class.simpleName, listTrainingPlanJson))

                    return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    fun getClassRecord(className: String?, json: String): String {
        return "$className***$json"
    }
}