package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerPackageController
import com.aitrainer.api.model.*
import com.aitrainer.api.repository.*
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class AppCustomerPackageTest {

    @Autowired
    private lateinit var customerRepository: CustomerRepository

    @Autowired
    private lateinit var customerExerciseDeviceRepository: CustomerExerciseDeviceRepository

    @Autowired
    private lateinit var exercisesRepository: ExercisesRepository

    @Autowired
    private lateinit var purchaseRepository: PurchaseRepository

    @Autowired
    private lateinit var customerPropertyRepository: CustomerPropertyRepository

    @Autowired
    private lateinit var exerciseResultRepository: ExerciseResultRepository

    @Autowired
    private lateinit var customerActivityRepository: CustomerActivityRepository

    @Autowired
    private lateinit var customerTrainingPlanRepository: CustomerTrainingPlanRepository

    @OptIn(ExperimentalStdlibApi::class)
    @Test
    fun customerPackageTest() {
        val gson = Gson()

        val controller = CustomerPackageController(customerRepository, customerExerciseDeviceRepository,
            exercisesRepository, purchaseRepository, customerPropertyRepository,
            exerciseResultRepository, customerActivityRepository, customerTrainingPlanRepository )
        var response: ResponseEntity<*> = controller.getCustomerPackageData(91)
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)

        response = controller.getCustomerPackageData(90)
        assertEquals(response.statusCode, HttpStatus.OK)

        val appPackageJson: String = response.body as String
        assertTrue(appPackageJson.isNotEmpty())


        val packages = appPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            when {
                record[0] == Customer::class.simpleName -> {
                    val customerJson: String = record[1]
                    val type = object : TypeToken<Customer?>(){}.type
                    val customer: Customer = gson.fromJson(customerJson, type)
                    assertEquals(customer.email, "sw@andio.biz")
                    assertEquals(customer.birthYear, 1972)
                    assertEquals(customer.trialDate, "2021-08-07 08:00:00")
                    assertEquals(customer.syncedDate, "2021-08-06 08:00:00")
                    assertEquals(customer.firebaseRegToken, "123456789asdfghjkl")
                    assertEquals(customer.lang!!.uppercase(), "HU")
                }
                record[0] == Exercises::class.simpleName -> {
                    val exercisesJson: String = record[1]
                    val type = object : TypeToken<List<Exercises?>?>() {}.type
                    val exerciseList: List<Exercises> = gson.fromJson(exercisesJson, type)
                    assertTrue(exerciseList.isNotEmpty())
                    assertEquals(exerciseList[0].exerciseTypeId, 2)
                    assertEquals(exerciseList[0].quantity, 34.0)
                }
                record[0] == CustomerExerciseDevice::class.simpleName -> {
                    val exerciseDeviceJson: String = record[1]
                    val type = object : TypeToken<List<CustomerExerciseDevice?>?>() {}.type
                    val customerExerciseDeviceList: List<CustomerExerciseDevice> = gson.fromJson(exerciseDeviceJson, type)
                    assertTrue(customerExerciseDeviceList.isNotEmpty())
                    assertEquals(customerExerciseDeviceList[1].exerciseDeviceId,2)
                }

                record[0] == CustomerProperty::class.simpleName -> {
                    //actual values
                    val propertyJson: String = record[1]
                    val type = object : TypeToken<List<CustomerProperty?>?>() {}.type
                    val propertyList: List<CustomerProperty> = gson.fromJson(propertyJson, type)
                    assertTrue(propertyList.isNotEmpty())
                    assertEquals(propertyList[2].propertyId,4)
                    assertEquals(propertyList[2].propertyValue,37.0)
                }
                record[0] == CustomerProperty::class.simpleName+"All" -> {
                    val propertyJson: String = record[1]
                    val type = object : TypeToken<List<CustomerProperty?>?>() {}.type
                    val propertyList: List<CustomerProperty> = gson.fromJson(propertyJson, type)
                    assertTrue(propertyList.isNotEmpty())
                    assertEquals(propertyList[1].propertyId,1)
                    assertEquals(propertyList[1].propertyValue,82.0)
                }
                record[0] == CustomerActivity::class.simpleName+"All" -> {
                    val activityJson: String = record[1]
                    val type = object : TypeToken<List<CustomerActivity?>?>() {}.type
                    val activityList: List<CustomerActivity> = gson.fromJson(activityJson, type)
                    assertTrue(activityList.isNotEmpty())
                    assertEquals(activityList[0].type,"basic_tutorial")
                }
                record[0] == CustomerTrainingPlan::class.simpleName+"All" -> {
                    val trainingPlanJson: String = record[1]
                    val type = object : TypeToken<List<CustomerTrainingPlan?>?>() {}.type
                    val trainingPlanList: List<CustomerTrainingPlan> = gson.fromJson(trainingPlanJson, type)
                    assertTrue(trainingPlanList.isNotEmpty())
                    assertEquals(trainingPlanList[0].customerId,91)
                    assertEquals(trainingPlanList[0].trainingPlanId,2)
                    assertEquals(trainingPlanList[0].details.size,2)
                    assertEquals(trainingPlanList[0].details[0].exerciseTypeId,60)
                    assertEquals(trainingPlanList[0].details[0].weight,24.0)
                }
            }
        }

    }

}